/*
Queries the database to find podcasts that need to have their feed crawled for updates.
*/

package tasks

import (
	"context"
	"fmt"
	"log"
	"os"
	"time"

	models "code.locsi.com/locsi/api/models"
	"github.com/hibiken/asynq"
)

// A list of task types.
const (
	TypePodcastGetNeedsUpdate = "podcast:getNeedsUpdate"
)

func NewPodcastGetNeedsUpdateTask(podSeriesID int) (*asynq.Task, error) {
	return asynq.NewTask(TypePodcastGetNeedsUpdate, nil), nil
}

func HandlePodcastGetNeedsUpdate(ctx context.Context, t *asynq.Task) error {
	log.Print("Getting podcasts that need updates.")

	// Get Podcasts that need to be crawled again.  Limit to 500 at a time.
	var podSeries []models.PodcastSeries
	if err := models.DB.Select("id").Where("next_crawl < ?", time.Now()).Limit(500).Find(&podSeries).Error; err != nil {
		return fmt.Errorf("Reading from the database failed: %v", err.Error())
	}

	for _, pod := range podSeries {
		const redisAddr = "queue-redis-master:6379"
		client := asynq.NewClient(asynq.RedisClientOpt{Addr: redisAddr, Password: os.Getenv("REDIS_PASSWORD")})
		defer client.Close()
		task, err := NewUpdatePodcastFromFeedTask(pod.ID, "")
		if err != nil {
			log.Fatalf("could not create task: %v", err)
		}
		info, err := client.Enqueue(task, asynq.MaxRetry(0))
		if err != nil {
			log.Fatalf("could not enqueue task: %v", err)
		}
		log.Printf("enqueued task for %v: id=%s queue=%s", pod.ID, info.ID, info.Queue)
	}

	/*TODO: Either use cron to run this every hour or add a task here to do so.  It would be nice to not have to rely on an external service like CRON.
	Consider adding an initial startup task to main that checks to see if there is a pending task and if not add one.
	*/

	// TODO: Next add logic to updatefromfeed task to set a next crawl time in the db
	return nil
}
