/*
Updates a Podcast and it's associated episodes by grabbing and parsing it's RSS feed.
*/

package tasks

import (
	"context"
	"encoding/json"
	"errors"
	"fmt"
	"log"
	"strings"

	models "code.locsi.com/locsi/api/models"
	"github.com/hibiken/asynq"
	"github.com/mmcdole/gofeed"
	"gorm.io/gorm/clause"
)

// A list of task types.
const (
	TypeUpdatePodcastFromFeed = "updatePodcast:fromFeed"
)

type UpdatePodcastFromFeedPayload struct {
	PodcastSeriesID int
	ProvidedFeed    string
}

func NewUpdatePodcastFromFeedTask(podSeriesID int, providedFeed string) (*asynq.Task, error) {
	payload, err := json.Marshal(UpdatePodcastFromFeedPayload{PodcastSeriesID: podSeriesID, ProvidedFeed: providedFeed})
	if err != nil {
		return nil, err
	}
	return asynq.NewTask(TypeUpdatePodcastFromFeed, payload), nil
}

// Queries our database to find podcast series that need to have their RSS feeds crawled.  It then stores new episode information found in the database.
func HandleUpdatePodcastFromFeedTask(ctx context.Context, t *asynq.Task) error {
	var p UpdatePodcastFromFeedPayload
	if err := json.Unmarshal(t.Payload(), &p); err != nil {
		return fmt.Errorf("json.Unmarshal failed: %v", err)
	}
	log.Printf("Updating episodes for: PodcastSeriesID=%d", p.PodcastSeriesID)

	// Grab podcast series feeds from the database.
	var podSeries models.PodcastSeries
	if err := models.DB.Preload("Categories").Find(&podSeries, p.PodcastSeriesID).Error; err != nil {
		return fmt.Errorf("Reading from the database failed: %v", err.Error())
	}

	// Grab the RSS feed for this podcast.
	feed, err := GetPodcastFeed(p.ProvidedFeed, podSeries)

	if err != nil {
		fmt.Printf("Failed to get the feed for %v, %v.\n", podSeries.ID, podSeries.WebFeed)
		return fmt.Errorf(err.Error(), asynq.SkipRetry)
	} else {
		// Process the RSS feed data.
		tx := models.DB.Begin()
		podEpisodes := []models.PodcastEpisode{}
		for _, item := range feed.Items {
			// Update/Insert episodes in our database.
			fmt.Printf("Converting to episode.  Series id:%v, GUID:%v\n", podSeries.ID, item.GUID)
			podEpisode, err := ToPodEpisode(item, podSeries.ID)
			if err == nil {
				podEpisodes = append(podEpisodes, podEpisode)
			}
		}
		if len(podEpisodes) > 0 {
			fmt.Printf("Inserting %v episodes for %v\n", len(podEpisodes), podSeries.ID)
			tx.Clauses(clause.OnConflict{
				Columns:   []clause.Column{{Name: "guid"}},
				UpdateAll: true,
			}).CreateInBatches(&podEpisodes, 50)
		}

		// Update the Podcast Series data from the feed too.
		updatedPodSeries := ToUpdatedPodSeries(feed, podSeries)
		fmt.Printf("Updating series %v\n", podSeries.ID)
		// Update the Podcast Series
		tx.Save(&updatedPodSeries)
		// Update the Podcast Series Categories (removes any that exist in the db that are not in the feed)
		tx.Model(&updatedPodSeries).Association("Categories").Replace(updatedPodSeries.Categories)
		tx.Commit()
	}

	return nil
}

func GetPodcastFeed(ProvidedFeed string, podSeries models.PodcastSeries) (*gofeed.Feed, error) {
	// If a feed was provided as part of the payload, try and use that.
	fp := gofeed.NewParser()
	feed, err := fp.ParseString(ProvidedFeed)
	if err != nil {
		fmt.Printf("Failed to process the provided feed for %v.\n", podSeries.ID)
		// Try to grab the feed from URL instead.
		fp := gofeed.NewParser()
		feed, err = fp.ParseURL(podSeries.WebFeed)
	}
	return feed, err
}

func ToPodEpisode(item *gofeed.Item, podSeriesId int) (models.PodcastEpisode, error) {
	var podEpisode models.PodcastEpisode
	if len(item.Enclosures) > 0 && item.Enclosures[0].URL != "" {
		podEpisode := models.PodcastEpisode{Name: item.Title, GUID: item.GUID, ContentURL: item.Enclosures[0].URL, PodcastSeriesID: podSeriesId}
		if item.ITunesExt != nil && item.ITunesExt.Duration != "" {
			podEpisode.Duration = item.ITunesExt.Duration
		}
		if item.ITunesExt != nil && item.ITunesExt.Image != "" {
			podEpisode.ImageURL = item.ITunesExt.Image
		}
		if item.PublishedParsed != nil {
			podEpisode.DatePublished = *item.PublishedParsed
		}
		if item.Content != "" {
			podEpisode.Description = item.Content
		} else if item.ITunesExt != nil && item.ITunesExt.Summary != "" {
			podEpisode.Description = item.ITunesExt.Summary
		}
		return podEpisode, nil

	} else {
		return podEpisode, errors.New("There is no content for this item.")
	}
}

func ToUpdatedPodSeries(feed *gofeed.Feed, podSeries models.PodcastSeries) models.PodcastSeries {
	// Compare the feed data to the existing podcast series in the database and update with any new data.
	updatedPodSeries := podSeries
	if feed.Title != "" && feed.Title != podSeries.Name {
		updatedPodSeries.Name = feed.Title
	}
	if feed.Link != "" && feed.Link != podSeries.URL {
		updatedPodSeries.URL = feed.Link
	}
	if feed.ITunesExt != nil && feed.ITunesExt.Explicit != "" {
		if (strings.ToLower(feed.ITunesExt.Explicit) == "yes" || strings.ToLower(feed.ITunesExt.Explicit) == "explicit" || strings.ToLower(feed.ITunesExt.Explicit) == "true") && podSeries.Explicit == false {
			updatedPodSeries.Explicit = true
		} else if (strings.ToLower(feed.ITunesExt.Explicit) == "no" || strings.ToLower(feed.ITunesExt.Explicit) == "clean" || strings.ToLower(feed.ITunesExt.Explicit) == "false") && podSeries.Explicit == true {
			updatedPodSeries.Explicit = false
		}
	}
	if feed.Image != nil && feed.Image.URL != "" && feed.Image.URL != podSeries.ImageURL {
		updatedPodSeries.ImageURL = feed.Image.URL
	}
	if feed.Language != "" && feed.Language != podSeries.InLanguage {
		updatedPodSeries.InLanguage = feed.Language
	}
	if feed.Description != "" && feed.Description != podSeries.Description {
		updatedPodSeries.Description = feed.Description
	}
	if feed.Extensions != nil {
		for _, ext := range feed.Extensions["atom"]["link"] {
			if ext.Attrs["rel"] == "self" && ext.Attrs["href"] != podSeries.WebSubSelf {
				updatedPodSeries.WebSubSelf = ext.Attrs["href"]
			}
			if ext.Attrs["rel"] == "hub" && ext.Attrs["href"] != podSeries.WebSubHub {
				updatedPodSeries.WebSubHub = ext.Attrs["href"]
			}
		}
	}

	// Update the categories for this podcast series.
	podCategories := []models.Categories{}
	// Add all of the categories in the feed to the db.
	if feed.Categories != nil {
		for _, cat := range feed.Categories {
			podCategories = append(podCategories, models.Categories{Category: strings.ToLower(cat)})
			if len(podCategories) == 10 {
				// Don't allow more than 10 categories for a Podcast Series
				break
			}
		}
		updatedPodSeries.Categories = podCategories
	}

	return updatedPodSeries
}
