package tasks

import (
	"fmt"
	"testing"
	"time"

	models "code.locsi.com/locsi/api/models"
	"github.com/mmcdole/gofeed"
)

func TestToPodEpisode(t *testing.T) {
	type expectedResult struct {
		value   models.PodcastEpisode
		isValid bool
	}
	parsedTime1, _ := time.Parse(time.RFC3339, "2015-05-29T22:59:00Z")
	parsedTime2, _ := time.Parse(time.RFC3339, "0000-00-00T00:00:00Z")
	var tests = []struct {
		podSeriesId    int
		episodeGUID    string
		expectedResult expectedResult
	}{
		{6, "tag:blogger.com,1999:blog-8429222294807394803.post-5283595917266045004", expectedResult{value: models.PodcastEpisode{
			Name: "June 6-7 2015 Dual Sport Events", DatePublished: parsedTime1, GUID: "tag:blogger.com,1999:blog-8429222294807394803.post-5283595917266045004", Duration: "",
			Description: "<iframe allowfullscreen=\"\" frameborder=\"0\" height=\"30\" mozallowfullscreen=\"true\" src=\"https://archive.org/embed/DualSport662015\" webkitallowfullscreen=\"true\" width=\"500\"></iframe> <enclosure length=\"14,339,473\" type=\"audio/mpeg\" url=\"https://archive.org/embed/DualSport662015\"><a href=\"http://archive.org/details/mark_usdualsports_Fdx\">Link</a></enclosure> <a href=\"http://ia601505.us.archive.org/4/items/DualSport662015/Dual%20Sport%206-6-2015.mp3\">Link</a>",
			ImageURL:    "", ContentURL: "http://ia601505.us.archive.org/4/items/DualSport662015/Dual%20Sport%206-6-2015.mp3", PodcastSeriesID: 6}, isValid: true}},
		{8, "f9b0051099697331aa933922a1cb6739", expectedResult{isValid: false}},
		{120, "f9b0051099697331aa933922a1cb6739", expectedResult{isValid: false}},
		{160, "http://ciep.mx/wp-content/uploads/2015/02/Podcast2_2.mp3", expectedResult{value: models.PodcastEpisode{
			Name: "Podcast CIEP | #2", DatePublished: parsedTime2, GUID: "http://ciep.mx/wp-content/uploads/2015/02/Podcast2_2.mp3", Duration: "5:30",
			Description: "Ajuste del gasto público: por qué se tiene que recortar el gasto y dónde estarán los recortes. Finalmente, se analiza la evolución del presupuesto asignado a la función Protección social y la improbabilidad de alcanzar una pensión universal.",
			ImageURL:    "", ContentURL: "http://ciep.mx/wp-content/uploads/2015/02/Podcast2_2.mp3", PodcastSeriesID: 160}, isValid: true}},
	}

	models.ConnectDatabase()
	for _, tt := range tests {
		var podcast_series models.PodcastSeries
		testname := fmt.Sprintf("Testing id %v\n", tt.episodeGUID)
		t.Run(testname, func(t *testing.T) {
			// Get a podcast from the db.
			if err := models.DB.Find(&podcast_series, int(tt.podSeriesId)).Error; err != nil {
				t.Errorf("Failed to get podcast from db: %v", err.Error())
			}
			// Grab the RSS feed for this podcast.
			fp := gofeed.NewParser()
			feed, err := fp.ParseURL(podcast_series.WebFeed)
			if err != nil {
				t.Errorf("Failed to get the feed for %s: %v", podcast_series.WebFeed, err.Error())
			} else {
				// Process the RSS feed data.
				for _, item := range feed.Items {
					if item.GUID == tt.episodeGUID {
						// This is the GUID we are trying to test.
						podEpisode, err := ToPodEpisode(item, tt.podSeriesId)
						if err != nil {
							if tt.expectedResult.isValid == true {
								t.Errorf("Failed to convert item to episode for %s, %s: %v", podcast_series.WebFeed, tt.episodeGUID, err.Error())
							}
						} else {
							if podEpisode.Name != tt.expectedResult.value.Name {
								t.Errorf("for %s, field Name: got %v, want %v", tt.episodeGUID, podEpisode.Name, tt.expectedResult.value.Name)
							}
							if podEpisode.DatePublished != tt.expectedResult.value.DatePublished {
								t.Errorf("for %s, field DatePublished: got %v, want %v", tt.episodeGUID, podEpisode.DatePublished, tt.expectedResult.value.DatePublished)
							}
							if podEpisode.GUID != tt.expectedResult.value.GUID {
								t.Errorf("for %s, field GUID: got %v, want %v", tt.episodeGUID, podEpisode.GUID, tt.expectedResult.value.GUID)
							}
							if podEpisode.Duration != tt.expectedResult.value.Duration {
								t.Errorf("for %s, field Duration: got %v, want %v", tt.episodeGUID, podEpisode.Duration, tt.expectedResult.value.Duration)
							}
							if podEpisode.Description != tt.expectedResult.value.Description {
								t.Errorf("for %s, field Description: got %v, want %v", tt.episodeGUID, podEpisode.Description, tt.expectedResult.value.Description)
							}
							if podEpisode.ImageURL != tt.expectedResult.value.ImageURL {
								t.Errorf("for %s, field ImageURL: got %v, want %v", tt.episodeGUID, podEpisode.ImageURL, tt.expectedResult.value.ImageURL)
							}
							if podEpisode.ContentURL != tt.expectedResult.value.ContentURL {
								t.Errorf("for %s, field ContentURL: got %v, want %v", tt.episodeGUID, podEpisode.ContentURL, tt.expectedResult.value.ContentURL)
							}
							if podEpisode.PodcastSeriesID != tt.expectedResult.value.PodcastSeriesID {
								t.Errorf("for %s, field PodcastSeriesID: got %v, want %v", tt.episodeGUID, podEpisode.PodcastSeriesID, tt.expectedResult.value.PodcastSeriesID)
							}
						}
					}
				}
			}
		})

	}
}

func TestToUpdatedPodSeries(t *testing.T) {
	var tests = []PodcastSeriesTest{
		{693423, expectedPodcastSeriesResult{value: models.PodcastSeries{ID: 693423, WebFeed: "https://lexfridman.com/feed/podcast/", Name: "Lex Fridman Podcast", URL: "https://lexfridman.com/", Explicit: false,
			ImageURL: "https://lexfridman.com/wordpress/wp-content/uploads/powerpress/artwork_3000-230.png", ITunesType: "episodic", InLanguage: "en-US", GUID: "89e87657-cd69-5a47-9190-1365d3de138d",
			Description: "Conversations about science, technology, history, philosophy and the nature of intelligence, consciousness, love, and power. Lex is an AI researcher at MIT and beyond.",
			WebSubHub:   "https://pubsubhubbub.appspot.com/", WebSubSelf: "https://lexfridman.com/feed/podcast/",
			Categories: []models.Categories{{Category: "technology"}, {Category: "science"}, {Category: "society & culture"}, {Category: "philosophy"}}}, isValid: true}},
		{943118, expectedPodcastSeriesResult{value: models.PodcastSeries{ID: 943118, WebFeed: "https://audioboom.com/channels/5093219.rss", Name: "The Tim Dillon Show", URL: "https://audioboom.com/channels/5093219", Explicit: false,
			ImageURL: "https://images.theabcdn.com/i/40034741/s=1400x1400/el=1/rt=fill.jpg", ITunesType: "episodic", InLanguage: "en-us", GUID: "b73a3f59-999b-548d-8f3d-c50fa4a754b1",
			Description: "Tim Dillon is a comedian and tour guide. He’s very excited to give you a tour of the end of the world. Each week from a porch in Los Angeles he shares apocalyptic visions with his friends and berates a local diner.",
			WebSubHub:   "https://pubsubhubbub.appspot.com/", WebSubSelf: "https://audioboom.com/channels/5093219.rss",
			Categories: []models.Categories{{Category: "comedy"}}}, isValid: true}},
	}

	for _, tt := range tests {
		testname := fmt.Sprintf("Testing series id %v\n", tt.ID)
		t.Run(testname, func(t *testing.T) {
			var podSeries models.PodcastSeries
			if err := models.DB.Preload("Categories").Find(&podSeries, tt.ID).Error; err != nil {
				t.Errorf("Failed to get podcast from db: %v", err.Error())
			}

			// Grab the RSS feed for this podcast.
			fp := gofeed.NewParser()
			feed, err := fp.ParseURL(podSeries.WebFeed)
			if err != nil {
				t.Errorf("Failed to get the feed for %s: %v", podSeries.WebFeed, err.Error())
			} else {
				updatedPodSeries := ToUpdatedPodSeries(feed, podSeries)

				// Run tests to validate the Podcast Series
				validatePodSeries(updatedPodSeries, tt, t)
			}
		})
	}
}

func TestGetPodcastFeed(t *testing.T) {
	var tests = []struct {
		podSeries    models.PodcastSeries
		ProvidedFeed string
		isValid      bool
	}{
		// Valid WebFeed and ProvidedFeed
		{models.PodcastSeries{ID: 691496, WebFeed: "https://feeds.simplecast.com/54nAGcIl"}, `<?xml version="1.0" encoding="UTF-8"?><feed xmlns="http://www.w3.org/2005/Atom"><status feed="https://feeds.simplecast.com/54nAGcIl" xmlns="http://superfeedr.com/xmpp-pubsub-ext"><http code="200">Fetched (ping) 200 172800 and parsed 1/218 entries</http><next_fetch>1970-01-20T10:07:24.373Z</next_fetch><entries_count_since_last_maintenance>2</entries_count_since_last_maintenance><velocity>1.5</velocity><generated_ids>true</generated_ids><title>The Daily</title><period>172800</period><last_fetch>1970-01-20T10:04:00.481Z</last_fetch><last_parse>1970-01-20T10:04:00.481Z</last_parse><last_maintenance_at>1970-01-20T10:01:08.278Z</last_maintenance_at></status><link title="MP3 Audio" rel="self" href="https://feeds.simplecast.com/54nAGcIl" type="application/atom+xml"/><link title="" rel="hub" href="https://simplecast.superfeedr.com/" type="text/html"/><link title="The Daily" rel="image" href="https://image.simplecastcdn.com/images/03d8b493-87fc-4bd1-931f-8a8e9b945d8a/2cce5659-f647-4366-b318-46e4b67afcfa/3000x3000/c81936f538106550b804e7e4fe2c236319bab7fba37941a6e8f7e5c3d3048b88fc5b2182fb790f7d446bdc820406456c94287f245db89d8656c105d5511ec3de.jpeg?aid=rss_feed"/><link title="The Daily" rel="alternate" href="https://www.nytimes.com/the-daily" type="text/html"/><title>The Daily</title><updated>2023-03-03T10:46:04.000Z</updated><id>the-daily-2023-3-3-10</id><entry xmlns="http://www.w3.org/2005/Atom" xmlns:geo="http://www.georss.org/georss" xmlns:as="http://activitystrea.ms/spec/1.0/" xmlns:sf="http://superfeedr.com/xmpp-pubsub-ext" xml:lang="en"><id>1b604c9e-11d9-44ed-ab2b-297438dea812</id><published>2023-03-03T10:45:00.000Z</published><updated>2023-03-03T10:45:00.000Z</updated><title>Why Russia Is Taking Thousands of Ukrainian Children</title><summary type="html">&lt;p&gt;As Russian troops pushed into Ukraine, children who were fleeing newly occupied territories were swept up. Many became part of a Russian effort to portray itself as a charitable savior. &lt;/p&gt;&lt;p&gt;The children were placed in Russian families and paraded on television. The Times interviewed one child who was taken from Ukraine, a girl named Anya, who said she ached to return.&lt;/p&gt;&lt;p&gt;Guest: &lt;a href="https://www.nytimes.com/by/emma-bubola?smid=pc-thedaily"&gt;Emma Bubola&lt;/a&gt;, a reporter for The New York Times based in London.&lt;/p&gt;&lt;p&gt;Background reading: &lt;/p&gt;&lt;ul&gt;&lt;li&gt;Using adoptions, Russia has turned Ukrainian children into&lt;a href="https://www.nytimes.com/2022/10/22/world/europe/ukraine-children-russia-adoptions.html#:~:text=Emma%20Bubola%2C%20a%20Times%20reporter,affected%20by%20Russia%27s%20resettlement%20policy.&amp;text=As%20Russian%20forces%20laid%20siege,group%20homes%20and%20boarding%20schools?smid=pc-thedaily"&gt; spoils of war&lt;/a&gt;.&lt;/li&gt;&lt;li&gt;The mayor of a Ukrainian city disappeared, but&lt;a href="https://www.nytimes.com/2023/03/02/world/europe/ukraine-kherson-mayor-russia.html"&gt; questions about his loyalty did not&lt;/a&gt;.&lt;/li&gt;&lt;/ul&gt;&lt;p&gt;For more information on today’s episode, visit &lt;a href="http://nytimes.com/thedaily?smid=pc-thedaily"&gt;nytimes.com/thedaily&lt;/a&gt;. Transcripts of each episode will be made available by the next workday. &lt;/p&gt;</summary><content type="html">&lt;p&gt;As Russian troops pushed into Ukraine, children who were fleeing newly occupied territories were swept up. Many became part of a Russian effort to portray itself as a charitable savior. &lt;/p&gt;&lt;p&gt;The children were placed in Russian families and paraded on television. The Times interviewed one child who was taken from Ukraine, a girl named Anya, who said she ached to return.&lt;/p&gt;&lt;p&gt;Guest: &lt;a href="https://www.nytimes.com/by/emma-bubola?smid=pc-thedaily"&gt;Emma Bubola&lt;/a&gt;, a reporter for The New York Times based in London.&lt;/p&gt;&lt;p&gt;Background reading: &lt;/p&gt;&lt;ul&gt;&lt;li&gt;Using adoptions, Russia has turned Ukrainian children into&lt;a href="https://www.nytimes.com/2022/10/22/world/europe/ukraine-children-russia-adoptions.html#:~:text=Emma%20Bubola%2C%20a%20Times%20reporter,affected%20by%20Russia%27s%20resettlement%20policy.&amp;text=As%20Russian%20forces%20laid%20siege,group%20homes%20and%20boarding%20schools?smid=pc-thedaily"&gt; spoils of war&lt;/a&gt;.&lt;/li&gt;&lt;li&gt;The mayor of a Ukrainian city disappeared, but&lt;a href="https://www.nytimes.com/2023/03/02/world/europe/ukraine-kherson-mayor-russia.html"&gt; questions about his loyalty did not&lt;/a&gt;.&lt;/li&gt;&lt;/ul&gt;&lt;p&gt;For more information on today’s episode, visit &lt;a href="http://nytimes.com/thedaily?smid=pc-thedaily"&gt;nytimes.com/thedaily&lt;/a&gt;. Transcripts of each episode will be made available by the next workday. &lt;/p&gt;</content><link title="Why Russia Is Taking Thousands of Ukrainian Children" rel="alternate" href="https://www.nytimes.com/the-daily" type="text/html" xml:lang="en"/><link title="Why Russia Is Taking Thousands of Ukrainian Children" rel="enclosure" href="https://dts.podtrac.com/redirect.mp3/chrt.fm/track/8DB4DB/pdst.fm/e/pfx.vpixl.com/6qj4J/nyt.simplecastaudio.com/03d8b493-87fc-4bd1-931f-8a8e9b945d8a/episodes/61da8396-12ae-4308-998f-68308dc3b70a/audio/128/default.mp3?aid=rss_feed&amp;awCollectionId=03d8b493-87fc-4bd1-931f-8a8e9b945d8a&amp;awEpisodeId=61da8396-12ae-4308-998f-68308dc3b70a&amp;feed=54nAGcIl" type="audio/mpeg" xml:lang="en"/><author xml:lang="en"><name>thedaily@nytimes.com (The New York Times)</name><uri></uri><email></email><id>thedaily-nytimes-com-the-new-york-times-</id></author><author xml:lang="en"><name>The New York Times</name><uri></uri><email></email><id>The New York Times</id></author></entry></feed>`,
			true},
		// Invalid ProvidedFeed and WebFeed
		{models.PodcastSeries{ID: 691496, WebFeed: "https://feeds.simplecast.com/invalid-url"},
			``,
			false},
		// Invalid ProvidedFeed and Valid WebFeed
		{models.PodcastSeries{ID: 691496, WebFeed: "https://feeds.simplecast.com/54nAGcIl"},
			``,
			true},
		// Invalid WebFeed and Valid ProvidedFeed
		{models.PodcastSeries{ID: 691496, WebFeed: "https://feeds.simplecast.com/invalid-url"}, `<?xml version="1.0" encoding="UTF-8"?><feed xmlns="http://www.w3.org/2005/Atom"><status feed="https://feeds.simplecast.com/54nAGcIl" xmlns="http://superfeedr.com/xmpp-pubsub-ext"><http code="200">Fetched (ping) 200 172800 and parsed 1/218 entries</http><next_fetch>1970-01-20T10:07:24.373Z</next_fetch><entries_count_since_last_maintenance>2</entries_count_since_last_maintenance><velocity>1.5</velocity><generated_ids>true</generated_ids><title>The Daily</title><period>172800</period><last_fetch>1970-01-20T10:04:00.481Z</last_fetch><last_parse>1970-01-20T10:04:00.481Z</last_parse><last_maintenance_at>1970-01-20T10:01:08.278Z</last_maintenance_at></status><link title="MP3 Audio" rel="self" href="https://feeds.simplecast.com/54nAGcIl" type="application/atom+xml"/><link title="" rel="hub" href="https://simplecast.superfeedr.com/" type="text/html"/><link title="The Daily" rel="image" href="https://image.simplecastcdn.com/images/03d8b493-87fc-4bd1-931f-8a8e9b945d8a/2cce5659-f647-4366-b318-46e4b67afcfa/3000x3000/c81936f538106550b804e7e4fe2c236319bab7fba37941a6e8f7e5c3d3048b88fc5b2182fb790f7d446bdc820406456c94287f245db89d8656c105d5511ec3de.jpeg?aid=rss_feed"/><link title="The Daily" rel="alternate" href="https://www.nytimes.com/the-daily" type="text/html"/><title>The Daily</title><updated>2023-03-03T10:46:04.000Z</updated><id>the-daily-2023-3-3-10</id><entry xmlns="http://www.w3.org/2005/Atom" xmlns:geo="http://www.georss.org/georss" xmlns:as="http://activitystrea.ms/spec/1.0/" xmlns:sf="http://superfeedr.com/xmpp-pubsub-ext" xml:lang="en"><id>1b604c9e-11d9-44ed-ab2b-297438dea812</id><published>2023-03-03T10:45:00.000Z</published><updated>2023-03-03T10:45:00.000Z</updated><title>Why Russia Is Taking Thousands of Ukrainian Children</title><summary type="html">&lt;p&gt;As Russian troops pushed into Ukraine, children who were fleeing newly occupied territories were swept up. Many became part of a Russian effort to portray itself as a charitable savior. &lt;/p&gt;&lt;p&gt;The children were placed in Russian families and paraded on television. The Times interviewed one child who was taken from Ukraine, a girl named Anya, who said she ached to return.&lt;/p&gt;&lt;p&gt;Guest: &lt;a href="https://www.nytimes.com/by/emma-bubola?smid=pc-thedaily"&gt;Emma Bubola&lt;/a&gt;, a reporter for The New York Times based in London.&lt;/p&gt;&lt;p&gt;Background reading: &lt;/p&gt;&lt;ul&gt;&lt;li&gt;Using adoptions, Russia has turned Ukrainian children into&lt;a href="https://www.nytimes.com/2022/10/22/world/europe/ukraine-children-russia-adoptions.html#:~:text=Emma%20Bubola%2C%20a%20Times%20reporter,affected%20by%20Russia%27s%20resettlement%20policy.&amp;text=As%20Russian%20forces%20laid%20siege,group%20homes%20and%20boarding%20schools?smid=pc-thedaily"&gt; spoils of war&lt;/a&gt;.&lt;/li&gt;&lt;li&gt;The mayor of a Ukrainian city disappeared, but&lt;a href="https://www.nytimes.com/2023/03/02/world/europe/ukraine-kherson-mayor-russia.html"&gt; questions about his loyalty did not&lt;/a&gt;.&lt;/li&gt;&lt;/ul&gt;&lt;p&gt;For more information on today’s episode, visit &lt;a href="http://nytimes.com/thedaily?smid=pc-thedaily"&gt;nytimes.com/thedaily&lt;/a&gt;. Transcripts of each episode will be made available by the next workday. &lt;/p&gt;</summary><content type="html">&lt;p&gt;As Russian troops pushed into Ukraine, children who were fleeing newly occupied territories were swept up. Many became part of a Russian effort to portray itself as a charitable savior. &lt;/p&gt;&lt;p&gt;The children were placed in Russian families and paraded on television. The Times interviewed one child who was taken from Ukraine, a girl named Anya, who said she ached to return.&lt;/p&gt;&lt;p&gt;Guest: &lt;a href="https://www.nytimes.com/by/emma-bubola?smid=pc-thedaily"&gt;Emma Bubola&lt;/a&gt;, a reporter for The New York Times based in London.&lt;/p&gt;&lt;p&gt;Background reading: &lt;/p&gt;&lt;ul&gt;&lt;li&gt;Using adoptions, Russia has turned Ukrainian children into&lt;a href="https://www.nytimes.com/2022/10/22/world/europe/ukraine-children-russia-adoptions.html#:~:text=Emma%20Bubola%2C%20a%20Times%20reporter,affected%20by%20Russia%27s%20resettlement%20policy.&amp;text=As%20Russian%20forces%20laid%20siege,group%20homes%20and%20boarding%20schools?smid=pc-thedaily"&gt; spoils of war&lt;/a&gt;.&lt;/li&gt;&lt;li&gt;The mayor of a Ukrainian city disappeared, but&lt;a href="https://www.nytimes.com/2023/03/02/world/europe/ukraine-kherson-mayor-russia.html"&gt; questions about his loyalty did not&lt;/a&gt;.&lt;/li&gt;&lt;/ul&gt;&lt;p&gt;For more information on today’s episode, visit &lt;a href="http://nytimes.com/thedaily?smid=pc-thedaily"&gt;nytimes.com/thedaily&lt;/a&gt;. Transcripts of each episode will be made available by the next workday. &lt;/p&gt;</content><link title="Why Russia Is Taking Thousands of Ukrainian Children" rel="alternate" href="https://www.nytimes.com/the-daily" type="text/html" xml:lang="en"/><link title="Why Russia Is Taking Thousands of Ukrainian Children" rel="enclosure" href="https://dts.podtrac.com/redirect.mp3/chrt.fm/track/8DB4DB/pdst.fm/e/pfx.vpixl.com/6qj4J/nyt.simplecastaudio.com/03d8b493-87fc-4bd1-931f-8a8e9b945d8a/episodes/61da8396-12ae-4308-998f-68308dc3b70a/audio/128/default.mp3?aid=rss_feed&amp;awCollectionId=03d8b493-87fc-4bd1-931f-8a8e9b945d8a&amp;awEpisodeId=61da8396-12ae-4308-998f-68308dc3b70a&amp;feed=54nAGcIl" type="audio/mpeg" xml:lang="en"/><author xml:lang="en"><name>thedaily@nytimes.com (The New York Times)</name><uri></uri><email></email><id>thedaily-nytimes-com-the-new-york-times-</id></author><author xml:lang="en"><name>The New York Times</name><uri></uri><email></email><id>The New York Times</id></author></entry></feed>`,
			true},
	}

	for _, tt := range tests {
		testname := fmt.Sprintf("Testing feed for series id %v\n", tt.podSeries.ID)
		t.Run(testname, func(t *testing.T) {
			feed, err := GetPodcastFeed(tt.ProvidedFeed, tt.podSeries)
			if err != nil {
				if tt.isValid == true {
					t.Errorf("Failed to get the feed for %d: %v", tt.podSeries.ID, err.Error())
				}
			} else {
				// See if this is a valid feed that has items.
				if len(feed.Items) == 0 {
					if tt.isValid == true {
						t.Errorf("There were no items in the feed for %d: %v", tt.podSeries.ID, err.Error())
					}
				} else if tt.isValid == false {
					t.Errorf("A valid feed was found and processed for %d", tt.podSeries.ID)
				}
			}
		})
	}
}
