/*
Checks the database for podcasts that support websub and either have a subscription that is expiring soon or don't have a subscription.  It then (re)subscribes those podcasts using WebSub.
*/

package tasks

import (
	"bytes"
	"context"
	"fmt"
	"log"
	"math/rand"
	"net/http"
	"net/url"
	"time"

	models "code.locsi.com/locsi/api/models"
	"github.com/hibiken/asynq"
)

// A list of task types.
const (
	TypeWebSubSubscribe = "webSub:subscribe"
)

func NewWebSubSubscribeTask() (*asynq.Task, error) {
	return asynq.NewTask(TypeWebSubSubscribe, nil), nil
}

func HandleWebSubSubscribeTask(ctx context.Context, t *asynq.Task) error {
	log.Printf("Finding podcasts to subscribe to at %s.", time.Now())

	// Grab podcast series feeds from the database that expire in less than two hours or don't have expiration set yet.
	var podSeries []models.PodcastSeries
	if err := models.DB.Where("web_sub_hub IS NOT NULL AND web_sub_self IS NOT NULL AND (web_sub_lease_expiration IS NULL OR web_sub_lease_expiration < ?)", time.Now().Add(time.Hour*time.Duration(2))).Find(&podSeries).Error; err != nil {
		return fmt.Errorf("Reading from the database failed: %v", err.Error())
	}

	// Subscribe to these podcasts using WebSub.
	for _, pod := range podSeries {
		log.Printf("P: %v.", pod.ID)
		makeSubscriptionRequest(pod)
	}
	return nil
}

func makeSubscriptionRequest(podSeries models.PodcastSeries) {
	log.Println("Subscribing to", podSeries.WebSubSelf)

	callbackUrl := "https://websub.locsi.com/sub/callback/" + fmt.Sprint(podSeries.ID) + "/"
	log.Println("Callback url:", callbackUrl)

	// See if we have a hub.secret for this podcast
	if podSeries.WebSubSecret == "" {
		// Create a hub secret for this podcast
		podSeries.WebSubSecret = getRandString(15)
		// Store the secret in the database
		models.DB.Save(&podSeries)
	}

	body := url.Values{}
	body.Set("hub.callback", callbackUrl)
	body.Add("hub.topic", podSeries.WebSubSelf)
	body.Add("hub.mode", "subscribe")
	body.Add("hub.secret", podSeries.WebSubSecret)

	req, _ := http.NewRequest("POST", podSeries.WebSubHub, bytes.NewBufferString(body.Encode()))
	req.Header.Add("Content-Type", "application/x-www-form-urlencoded")
	req.Header.Add("From", "Locsi WebSub")

	client := &http.Client{}
	resp, err := client.Do(req)

	if err != nil {
		log.Printf("Subscription failed, %v, %s", podSeries.ID, err)

	} else if resp.StatusCode != 202 {
		log.Printf("Subscription failed, %v, status = %s", podSeries.ID, resp.Status)
	}
}

func getRandString(n int) string {
	rand.Seed(time.Now().UnixNano())
	const letterBytes = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"

	b := make([]byte, n)
	for i := range b {
		b[i] = letterBytes[rand.Intn(len(letterBytes))]
	}
	return string(b)
}
